import bpy
import blf
from bpy.props import (IntProperty,
                       FloatProperty,
                       )


def draw_pipe_options(self, context):
    if context.object.type == 'CURVE':
        segments = context.object.data.bevel_resolution
        pipe_width = context.object.data.bevel_depth
        width = context.area.width
        font_id = 0
        blf.color(font_id, 1, 1, 1, 1)
        blf.position(font_id, width / 2 - 100, 150, 0)
        blf.size(font_id, 30, 72)
        blf.draw(font_id, "Segments: ")
        blf.position(font_id, width / 2 - 100, 100, 0)
        blf.draw(font_id, "Width: ")

        blf.color(font_id, 253, 253, 0, 1)
        blf.position(font_id, width / 2 + 66, 150, 0)
        blf.size(font_id, 30, 72)
        blf.draw(font_id, str(4 + segments * 2))
        blf.position(font_id, width / 2 + 3, 100, 0)
        blf.draw(font_id, str("%.3f" % pipe_width))


def check_selection(context):
    bpy.ops.object.mode_set(mode='OBJECT')
    selected_edges = []
    for obj in context.selected_objects:
        context.view_layer.objects.active = obj
        for e in context.active_object.data.edges:
            if e.select is True:
                selected_edges.append(e)
    bpy.ops.object.mode_set(mode='EDIT')
    if len(selected_edges) > 0:
        return True


class jmPipeTool(bpy.types.Operator):
    bl_idname = "object.quickpipe"
    bl_label = "Quick Pipe"
    bl_options = {'REGISTER', 'UNDO'}

    first_mouse_x: IntProperty()
    first_value: FloatProperty()

    @classmethod
    def poll(cls, context):
        return context.object.mode == 'EDIT'

    def separate_curves(self, context):
        # Agnieszka Pas "Separate Curve Object"
        # https://github.com/agapas/separate-curves/blob/master/separate_curves.py
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.curve.select_all(action='DESELECT')
        splines = context.active_object.data.splines
        while len(splines) > 1:
            splinee = splines[0]
            if splinee.points:
                splinee.points[0].select = True
            bpy.ops.curve.select_linked()
            bpy.ops.curve.separate()
        bpy.ops.object.mode_set(mode='OBJECT')

    def modal(self, context, event):
        addon_prefs = context.preferences.addons["quick_pipe"].preferences

        if event.type in {'RIGHTMOUSE', 'LEFTMOUSE', 'SPACE'}:
            context.object.show_wire = False
            self.separate_curves(context)
            if addon_prefs.convert_to_mesh is True:
                bpy.ops.object.convert(target='MESH')
            bpy.ops.object.mode_set(mode='OBJECT')
            context.area.header_text_set(None)
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            return {'FINISHED'}

        if event.type == 'W' and event.value == "PRESS":
            context.object.show_wire = not context.object.show_wire
            return {'RUNNING_MODAL'}

        if event.type == 'S' and event.value == "PRESS":
            if context.object.data.splines[0].use_smooth is True:
                bpy.ops.object.shade_flat()
            else:
                bpy.ops.object.shade_smooth()

        if event.type == 'C':
            # Convert to mesh
            context.object.show_wire = False
            bpy.ops.object.convert(target='MESH')
            context.area.header_text_set(None)
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            return {'FINISHED'}

        if event.type == 'Q':
            context.object.show_wire = False
            self.separate_curves(context)

            # Match pipe width for each curve
            pipe_obj = [obj for obj in context.selected_objects]

            for obj in pipe_obj:
                bpy.ops.object.mode_set(mode='OBJECT')
                bpy.ops.object.select_all(action='DESELECT')
                obj.select_set(state=True)
                context.view_layer.objects.active = obj
                if context.object.type == 'CURVE':
                    bpy.ops.object.matchpipewidth()
            bpy.ops.object.mode_set(mode='OBJECT')
            # Select all pipes
            for obj in pipe_obj:
                obj.select_set(state=True)

            context.area.header_text_set(None)
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            return {'FINISHED'}

        if event.type == 'MIDDLEMOUSE':
            bpy.ops.view3d.rotate()
            return {'PASS_THROUGH'}

        if event.type == 'MIDDLEMOUSE' and event.value == "PRESS":
            bpy.ops.view3d.move()
            return {'PASS_THROUGH'}

        if event.type == 'MOUSEMOVE':
            delta = (self.first_mouse_x - event.mouse_x)

            if event.ctrl:
                delta *= 0.1

            context.object.data.bevel_depth = abs((self.first_value + delta) * 0.002)

        elif event.type == 'WHEELUPMOUSE':
            context.object.data.bevel_resolution += 1

        elif event.type == 'WHEELDOWNMOUSE':
            if context.object.data.bevel_resolution > 0:
                context.object.data.bevel_resolution -= 1

        segments = context.object.data.bevel_resolution
        pipe_width = context.object.data.bevel_depth

        context.area.header_text_set(
            "Width: (" + ("%.2f" % pipe_width) + "), Segments (" + str(4 + segments * 2) +
            "), MMB: roatate view, Shift+MMB: pan view, RMB/LMB/Space: confirm, WhUp/WhDown: for number of segments," +
            "Q: match pipe width, W: toggle wireframe, S: toggle smooth shading, C: convert to mesh"
        )
        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        addon_prefs = context.preferences.addons["quick_pipe"].preferences
        args = (self, context)
        self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_pipe_options, args, 'WINDOW', 'POST_PIXEL')

        if check_selection(context) is True:
            if context.object:
                if context.object.type == 'MESH':
                    self.first_mouse_x = event.mouse_x

                    source_objects = [obj for obj in context.selected_objects]
                    # Separate selected items
                    bpy.ops.mesh.duplicate_move()
                    bpy.ops.mesh.separate(type='SELECTED')
                    bpy.ops.object.mode_set(mode='OBJECT')

                    all_objects = [obj for obj in context.selected_objects]
                    # Find separated objects
                    separated_objects = [obj for obj in all_objects if obj not in source_objects]

                    bpy.ops.object.select_all(action='DESELECT')
                    # Select separated objects
                    for obj in separated_objects:
                        obj.select_set(state=True)

                    separated_objects[0].name = "Pipe"
                    context.view_layer.objects.active = separated_objects[0]
                    bpy.ops.object.join()

                    if addon_prefs.display_wire is True:
                        context.object.show_wire = True

                    bpy.ops.object.convert(target='CURVE')
                    if addon_prefs.enable_smooth_shading is True:
                        bpy.ops.object.shade_smooth()

                    pipe = context.view_layer.objects.active
                    pipe.data.fill_mode = 'FULL'
                    pipe.data.bevel_resolution = 1
                    pipe.data.bevel_depth = 0.1

                    self.first_value = pipe.data.bevel_depth

                    context.window_manager.modal_handler_add(self)
                    return {'RUNNING_MODAL'}

                else:
                    self.report({'WARNING'}, "No active object, could not finish")
                    return {'CANCELLED'}
        else:
            self.report({'INFO'}, 'Select edges')
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            return {'FINISHED'}
